import { AxiosError } from 'axios'
import { useMutation as useRQMutation, UseMutationOptions } from 'react-query'
import axiosService from 'src/services/axios-service'

export namespace DownloadFileMutation {
  export type Params = { filePath: string }

  export type Response = Blob

  export const key = ['downloadFile']

  export const getKey = (params: Params) => [...key, params]

  export const mutationFn = async (params: Params) => {
    const response = await axiosService.post(`/files/download`, params, {
      responseType: 'blob',
    })
    return response.data
  }

  export const useMutation = (
    options: UseMutationOptions<
      Response,
      AxiosError,
      DownloadFileMutation.Params
    > = {}
  ) => {
    return useRQMutation(mutationFn, { ...options, mutationKey: key })
  }
}
